#include "Arduino.h"
#include "I2Cdev.h"
#include "MPU6050_6Axis_MotionApps20.h"
#include "leds.h"
#include <avr/eeprom.h>

// ###############  Sensor - Einstellungen und Parameter  ###################

MPU6050 mpu;

// MPU control/status vars
bool dmpReady = false;  // set true if DMP init was successful
uint8_t mpuIntStatus;   // holds actual interrupt status byte from MPU
uint8_t devStatus;      // return status after each device operation (0 = success, !0 = error)
uint16_t packetSize;    // expected DMP packet size (default is 42 bytes)
uint16_t fifoCount;     // count of all bytes currently in FIFO
uint8_t fifoBuffer[64]; // FIFO storage buffer

// orientation/motion vars
Quaternion q;           // [w, x, y, z]         quaternion container
VectorInt16 aa;         // [x, y, z]            accel sensor measurements
VectorInt16 aaReal;     // [x, y, z]            gravity-free accel sensor measurements
VectorInt16 aaWorld;    // [x, y, z]            world-frame accel sensor measurements
VectorFloat gravity;    // [x, y, z]            gravity vector
float euler[3];         // [psi, theta, phi]    Euler angle container
float ypr[3];           // [yaw, pitch, roll]   yaw/pitch/roll container and gravity vector
VectorFloat omega;
int32_t omega_r[3];

// packet structure for InvenSense teapot demo
uint8_t teapotPacket[14] = { '$', 0x02, 0,0, 0,0, 0,0, 0,0, 0x00, 0x00, '\r', '\n' };

// #################################################################################################
// #################################################################################################

// ================================================================
// ===               INTERRUPT DETECTION ROUTINE                ===
// ================================================================

volatile bool mpuInterrupt = false;     // indicates whether MPU interrupt pin has gone high
void dmpDataReady() 
{
    mpuInterrupt = true;
}

void initialisiereMPU()
{
    #if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
        Wire.begin();
        TWBR = 24; // 400kHz I2C clock (200kHz if CPU is 8MHz)
    #elif I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE
        Fastwire::setup(400, true);
    #endif


    Serial.println(F("Initializing I2C devices..."));
    mpu.initialize();

    Serial.println(F("Testing device connections..."));
    Serial.println(mpu.testConnection() ? F("MPU6050 connection successful") : F("MPU6050 connection failed"));

    Serial.println(F("Initializing DMP..."));    // load and configure the DMP
    devStatus = mpu.dmpInitialize();
/*
    mpu.setXGyroOffset(220);    // supply your own gyro offsets here, scaled for min sensitivity
    mpu.setYGyroOffset(76);
    mpu.setZGyroOffset(-1);
    mpu.setXAccelOffset(-370); // 1688 factory default for my test chip
    mpu.setYAccelOffset(2115); // 1688 factory default for my test chip
    mpu.setZAccelOffset(2090); // 1688 factory default for my test chip
*/
    if (devStatus == 0) {                            // make sure it worked (returns 0 if so)
        Serial.println(F("Enabling DMP..."));        // turn on the DMP, now that it's ready
        mpu.setDMPEnabled(true);

        Serial.println(F("Enabling interrupt detection (Arduino external interrupt 0)..."));  // enable Arduino interrupt detection
        attachInterrupt(2, dmpDataReady, RISING);
        mpuIntStatus = mpu.getIntStatus();

        Serial.println(F("DMP ready! Waiting for first interrupt..."));     // set our DMP Ready flag so the main loop() function knows it's okay to use it
        dmpReady = true;

        packetSize = mpu.dmpGetFIFOPacketSize();  // get expected DMP packet size for later comparison
    } else {                                      // ERROR!    1 = initial memory load failed    2 = DMP configuration updates failed    (if it's going to break, usually the code will be 1)
        Serial.print(F("DMP Initialization failed (code "));
        Serial.print(devStatus);
        Serial.println(F(")"));
        while (1);                                // if programming failed, don't try to do anything
    }    
}

void checkMPUstatus()
{
    mpuInterrupt = false;                  // reset interrupt flag and get INT_STATUS byte
    mpuIntStatus = mpu.getIntStatus();
    fifoCount = mpu.getFIFOCount();        // get current FIFO count
  
    if ((mpuIntStatus & 0x10) || fifoCount == 1024) 
    {    
        // check for overflow (this should never happen unless our code is too inefficient)
        mpu.resetFIFO();                                 // reset so we can continue cleanly
//        Serial.println(F("FIFO overflow!"));
    } 
    else if (mpuIntStatus & 0x02)                      // otherwise, check for DMP data ready interrupt (this should happen frequently)
    {
        while (fifoCount < packetSize) fifoCount = mpu.getFIFOCount();  // wait for correct available data length, should be a VERY short wait
        mpu.getFIFOBytes(fifoBuffer, packetSize);        // read a packet from FIFO
        fifoCount -= packetSize;                         // track FIFO count here in case there is > 1 packet available (this lets us immediately read more without waiting for an interrupt)

        mpu.dmpGetQuaternion(&q, fifoBuffer);
        mpu.dmpGetGravity(&gravity, &q);
        mpu.dmpGetYawPitchRoll(ypr, &q, &gravity);
        mpu.resetFIFO();

       /*
        Serial.print("Fehler: \t");
        Serial.print("\t");
        Serial.print("STATE: \t");
        Serial.print("\t");
        Serial.println("");
       */
    }  
}
